@@ -1,18 +1,17 @@
 import json
 import logging
-import operator
 import os
 import time
 from enum import Flag
-from functools import reduce
+from threading import Event
 
 from pkg_resources import get_distribution
 
 import requests
 from attrdict import AttrDict
-from websocket import ABNF, WebSocket
 
-from .exceptions import ConnectionError, GatewayClosedException, HttpError
+from .exceptions import HttpError, NetworkError
+from .gateway import Gateway
 from .ratelimit import RateLimiter
 from .standard_listeners import add_standard_listeners
 
@@ -41,10 +40,15 @@ class Intent(Flag):
 
     @staticmethod
     def all():
-        return reduce(operator.ior, Intent.__members__.values())
+        return ~Intent(0)
+
+    @staticmethod
+    def unprivileged():
+        return ~(Intent.GUILD_PRESENCES | Intent.GUILD_MEMBERS)
 
 
 recoverable_error_codes = {
+    *range(1000, 1016),  # standard protocol error codes
     4000,  # unknown error
     4001,  # unknown opcode
     4002,  # decode error
@@ -59,7 +63,7 @@ class SmallD:
         self,
         token=os.environ.get("SMALLD_TOKEN"),
         base_url="https://discord.com/api/v6",
-        intents=Intent.all(),
+        intents=Intent.unprivileged(),
     ):
         if not token:
             raise ValueError("No bot token provided")
@@ -68,6 +72,7 @@ class SmallD:
         self.base_url = base_url
         self.intents = intents
         self.listeners = []
+        self.closed_event = Event()
 
         self.http = HttpClient(token, base_url)
 
@@ -77,6 +82,8 @@ class SmallD:
         self.patch = self.http.patch
         self.delete = self.http.delete
 
+        add_standard_listeners(self)
+
     def __getattr__(self, name):
         if name.startswith("on_"):
             return lambda: self.on_dispatch(t=name.strip("on_").upper())
@@ -109,52 +116,51 @@ class SmallD:
         logger.debug("gateway payload sent: %s", payload)
         self.gateway.send(payload)
 
+    @property
+    def closed(self):
+        return self.closed_event.is_set()
+
     def reconnect(self):
         self.gateway.close()
 
-    def run(self):
-        add_standard_listeners(self)
+    def close(self):
+        self.closed_event.set()
+        self.http.close()
+        self.reconnect()
 
-        while True:
+    def __enter__(self):
+        return self
+
+    def __exit__(self, *args):
+        if not self.closed:
+            self.close()
+
+    def run(self):
+        logger.info("Running (SmallD v%s)...", __version__)
+
+        self.closed_event.clear()
+
+        while not self.closed:
+            logger.info("Gateway connecting...")
             gateway_url = self.get("/gateway/bot").url
 
             self.gateway = Gateway(gateway_url)
 
-            try:
-                for data in self.gateway:
-                    logger.debug("gateway payload received: %s", data)
-                    for listener in self.listeners:
-                        listener(data)
-            except GatewayClosedException as e:
-                if e.code not in recoverable_error_codes:
-                    raise
+            for data in self.gateway:
+                logger.debug("gateway payload received: %s", data)
+                for listener in self.listeners:
+                    listener(data)
 
-            time.sleep(5)
+            reason = self.gateway.close_reason
+            if reason and not reason.code in recoverable_error_codes:
+                logger.fatal(
+                    "Unrecoverable gateway closure: %s", self.gateway.close_reason
+                )
+                self.close()
 
-
-class Gateway:
-    def __init__(self, url):
-        self.url = url
-        self.ws = WebSocket()
-
-    def __iter__(self):
-        self.ws.connect(self.url)
-
-        while self.ws.connected:
-            with self.ws.readlock:
-                opcode, data = self.ws.recv_data()
-
-            if data and opcode == ABNF.OPCODE_TEXT:
-                decoded_data = data.decode("utf-8")
-                yield AttrDict(json.loads(decoded_data))
-            elif data and opcode == ABNF.OPCODE_CLOSE:
-                raise GatewayClosedException.parse(data)
-
-    def send(self, data):
-        self.ws.send(data)
-
-    def close(self):
-        self.ws.close()
+            if not self.closed:
+                logger.info("Pausing before reconnect...")
+                time.sleep(5)
 
 
 class HttpClient:
@@ -203,7 +209,7 @@ class HttpClient:
         try:
             res = self.session.request(method, f"{self.base_url}/{path}", **args)
         except (requests.ConnectionError, requests.Timeout):
-            raise ConnectionError
+            raise NetworkError
         except requests.RequestException:
             raise HttpError
 
@@ -214,7 +220,7 @@ class HttpClient:
 
         try:
             content = res.json() if res.status_code != 204 else {}
-        except request.RequestException:
+        except json.JSONDecodeError:
             raise HttpError(response=res)
 
         return AttrDict(content)
